@@ -78,6 +78,7 @@ c -------------------------------------tracking a ptcl
 #include  "Zincidentv.h"
 #include  "Zcode.h"
 #include  "Zmanagerp.h"
+#include  "Zstackv.h"
 c                next is for DEBUG
 c//////
 c      logical deb
@@ -86,7 +87,7 @@ c/////////
 cc      cc#define DEBUG 1
 
 
-
+      logical reset
       real*8 cosangle
       character*70 msg
       integer nextwhere, never
@@ -133,7 +134,13 @@ c         (or reaches BorderHeightL)  If so, reset MovedTrack.
       call checkstat('after cifXObsSite')
 #endif
       if(Trace .ne. 0) then
-         call cputTrInfo        ! put trace info.
+         if( mod(Trace, 10) .le. 2) then
+            call cputTrInfo     ! put trace info.
+         elseif(mod(Trace,10) .le. 4) then
+            if( TrackBefMove.pos.height .le. HeightList(1)) then
+               call cputTrInfo  ! put trace info.
+            endif
+         endif
       endif
 #if DETAILED_TRACKING >= 3
 c            the user may kill the ptcl
@@ -173,11 +180,31 @@ c              stack current ptcl
          call cpush(MovedTrack)
       elseif(MoveStat .eq. ToInteract ) then
          if(Zfirst.pos.depth .eq. 0.) then
+            if(  (MovedTrack.p.code .ge. kpion  .and. 
+     *            MovedTrack.p.code .le. knuc ) .or.
+     *         MovedTrack.p.code .eq. kgnuc ) then
+c
+c                   from  v6.31 1ry is judged by Stack_pos
+c                  (even if this is not used, almost no problem)
+c                     ! if no stack, the particle should be 1ry (since Zfirst=0)
+               if( Stack_pos .eq. 0 .and.
+     *             IntInfArray(ProcessNo).process .eq. 'coll') then
+                  reset = .true.
+               else
+                  reset = .false.
+               endif
+            else
+               reset = .true.
+            endif
+            if(reset) then
 c             reset minimum time to reach the obs level (time from this
 c             hight to the obs. level), if no mag. exists until
 c                  the first collision point. or ptcl goes streight
-            call cresetTimer(MovedTrack)
-            Zfirst = MovedTrack
+c                  comment out from v6.31
+c               call cresetTimer(MovedTrack)
+
+               Zfirst = MovedTrack
+            endif
          endif
 #if DETAILED_TRACKING >= 1 
          call cobservation(4)
@@ -373,11 +400,9 @@ c         argon
       endif
 
       if(HowGeomag .le. 2 .or. HowGeomag .eq. 31) then
-c             distant; change of B is < 1 % 
+c             distant; change of B is < 1 % (for default
+c             MagChgDist =20 km; you can change it)
          call clengSmallBC(TrackBefMove, distant)
-c   **********
-         distant = distant/10.
-c   **********
          if( (TrackBefMove.pos.xyz.r(1) - x)**2 + 
      *        (TrackBefMove.pos.xyz.r(2) - y)**2 + 
      *        (TrackBefMove.pos.xyz.r(3) - z)**2 
